#ifndef _TRACKING_SYSTEMS_ENUM_H_
#define _TRACKING_SYSTEMS_ENUM_H_

/**************************************************************************
* Copyright (c) 2007
* Computer Aided Interventions and Medical Robotics (CAIMR) group,
* Imaging Science and Information Systems (ISIS) research center,
* Georgetown University.
*
* All rights reserved.
*
* Permission to use, copy, modify, distribute, and sell this software
* and its documentation for any purpose is hereby granted without fee,
* provided that:
* (1) Redistributions of source code must retain the above copyright notice,
*     this list of conditions and the following disclaimer.
* (2) Redistributions in binary form must reproduce the above copyright notice,
*     this list of conditions and the following disclaimer in the documentation
*      and/or other materials provided with the distribution.
* (3) The author's name may not be used to endorse or promote products derived
*     from this software without specific prior written permission.
* (4) Modified source versions must be plainly marked as such, and must not be
*     misrepresented as being the original software.
*
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDER ``AS IS'' AND ANY EXPRESS
* OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
* OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
* IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
* SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
* PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
* OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
* WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
* OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
* OF THE POSSIBILITY OF SUCH DAMAGE.
**************************************************************************/

#include <string>

/**
 * A strongely typed enumeration class, wraps a standard C++ enum.
 * Hopefully in the near future C++ enums will become type safe (see
 * C++ standards committee proposal "Strongly Typed Enums", SC22/WG21/N1719, 2004).
 *
 * @Author Ziv Yaniv
 */

class TrackingSystemsEnum {
public:
  /**
   * Values used as constants in switch statements.
   */
  enum {POLARIS_VICRA_VALUE=0,AURORA_VALUE};
  /**
   * Types of tracker systems.
   */
  static const TrackingSystemsEnum POLARIS_VICRA, AURORA;

  /**
   * Default constructor, results in an object representing the polaris system.
   */
  TrackingSystemsEnum() : value(POLARIS_VICRA_VALUE), description("polaris vicra") {} 

  /**
   * Copy constructor.
   */
  TrackingSystemsEnum(const TrackingSystemsEnum& other) : value(other.value), description(other.description) {}
  /**
   * Assignment operator.
   */
  TrackingSystemsEnum &operator=(const TrackingSystemsEnum& right) {this->value = right.value; this->description = right.description; return *this;}

  /**
   * Access the C++ enum value wrapped by this object.
   */
  int GetValue() const {return this->value;}

   /**
    * The string description of this enum. 
    */
  std::string ToString() const {return this->description;} 

  /**
   * Cast operator so that the TrackingSystemEnum objects can be used in comparisson
   * expressions (e.g. switch, if).
   */
  operator int() {return this->value;}

private:
  TrackingSystemsEnum(int v, std::string d) : value(v), description(d) {};
  int value;
  std::string description;
};
#endif //_TRACKING_SYSTEMS_ENUM_H_
